;===========================================================================================
; Trackbar.asm  Fasm Flat assembler
; Trackbar32 class
; pas de resource, controle total de control class
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  ID_GBOX       = 100
  ID_TRACKBARH1 = 101
  ID_TRACKBARH2 = 102
  ID_TRACKBARH3 = 103
  ID_TRACKBARV1 = 104
  ID_TRACKBARV2 = 105
  ID_TRACKBARV3 = 106
  ID_EDIT       = 107
  ID_SPIN       = 108

  UDM_SETRANGE32 = 046Fh
  UDM_SETPOS32   = 0471h

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

  start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU+\
                          WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande fin d'application
        ; -------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE
        ; Assure que la bibliothque de liaison dynamique de contrle commune est charge
        ; -------------------------------------------------------------------------------
        mov [icex.dwSize],sizeof.INITCOMMONCONTROLSEX
        mov [icex.dwICC],0FFFFh
        invoke InitCommonControlsEx,icex

        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      MSSansSerif
        mov [hFontMSSS],eax                      ;font handle

        ; Construit tous les controles
        ; ----------------------------
        stdcall CreateControls,[@hWnd]

        mov [NewPos],10

        stdcall TrackBarPos,[@hWnd]

    .elseif [@uMsg] = WM_HSCROLL | [@uMsg] = WM_VSCROLL
        mov eax,[@wParam]
        and eax,0FFFFh
        .if eax = TB_THUMBPOSITION
            mov eax,[@wParam]
            shr eax,16
            mov [NewPos],eax
            stdcall TrackBarPos,[@hWnd]
        .elseif eax = TB_THUMBTRACK
            mov eax,[@wParam]
            shr eax,16
            mov [NewPos],eax
            stdcall TrackBarPos,[@hWnd]
        .elseif eax = SB_LINEUP
            .if [NewPos] <> 0
                dec [NewPos]
            .endif
            stdcall TrackBarPos,[@hWnd]
        .elseif eax = SB_LINEDOWN
            .if [NewPos] <> 20
                inc [NewPos]
            .endif
            stdcall TrackBarPos,[@hWnd]
        .endif

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
proc CreateControls @hWnd

    ; Cration de la fenetre-enfant GroupBox
    ; --------------------------------------
    stdcall Dlg,Button,GbGges,[@hWnd],4,2,286,20,ID_GBOX,BS_GROUPBOX,0

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarH1
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],16,30,100,34,ID_TRACKBARH1,\
                                        WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH1
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarH2
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],16,80,100,34,ID_TRACKBARH2,\
                                TBS_TOP+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH2
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarH3
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],16,130,100,40,ID_TRACKBARH3,\
                                TBS_BOTH+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarH3
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarV1
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],240,50,34,100,ID_TRACKBARV1,\
                                TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0
    ; Min/max TrackBarV1
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarV2
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],190,50,34,100,ID_TRACKBARV2,\
                        TBS_TOP+TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0

    ; Min/max TrackBarV2
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant TrackbarV3
    ; ----------------------------------------
    stdcall Dlg,TrackBar,NoText,[@hWnd],130,50,40,100,ID_TRACKBARV3,\
                       TBS_BOTH+TBS_VERT+WS_TABSTOP+TBS_AUTOTICKS,0

    ; Min/max TrackBarV3
    ; ------------------
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETRANGEMIN,0,0
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETRANGEMAX,0,20

    ;=======================================================================================
    ; Cration de la fenetre-enfant Edit
    ; ----------------------------------
    stdcall Dlg,Edit,NoText,[@hWnd],131,166,38,20,ID_EDIT,ES_CENTER+ES_NUMBER+ES_READONLY,\
                                                                         WS_EX_CLIENTEDGE

    ;=======================================================================================
    ; Cration de la fenetre-enfant Spin
    ; ----------------------------------
    stdcall Dlg,Spin,NoText,[@hWnd],131,190,38,16,ID_SPIN,UDS_SETBUDDYINT+UDS_AUTOBUDDY+\
                                                               UDS_ARROWKEYS+UDS_HORZ,0
    ; Min/max Spin
    ; ------------
    invoke SendDlgItemMessage,[@hWnd],ID_SPIN,UDM_SETRANGE32,0,20
    ;=======================================================================================

    ret
endp

;===========================================================================================
; Cration d'une fenetre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AddrText = adresse du control text
; @hParent  = handle de la fenetre parent
; @X        = x-coordonne fenetre-enfant
; @Y        = y-coordonne fenetre-enfant
; @Xw       = largeur en pixels fenetre-enfant
; @Yh       = hauteur en pixels fenetre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; sortie eax = handle de la fenetre-enfant
;-------------------------------------------------------------------------------------------
proc Dlg @Class, @AddrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AddrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMSSS],NULL
    mov eax,ebx

    ret
endp

;===========================================================================================
proc TrackBarPos @hWnd

    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH1,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH2,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARH3,TBM_SETPOS,-1,[NewPos]

    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV1,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV2,TBM_SETPOS,-1,[NewPos]
    invoke SendDlgItemMessage,[@hWnd],ID_TRACKBARV3,TBM_SETPOS,-1,[NewPos]

    invoke SendDlgItemMessage,[@hWnd],ID_SPIN,UDM_SETPOS32,0,[NewPos]

    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Titre       db ' TRACKBAR32 control',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0
  GbGges      db 'AsmGges TrackBar32',0
  NoText      db '',0


  MSSansSerif db 'MS Sans Serif',0

  Edit        db 'EDIT',0
  Button      db 'BUTTON',0
  TrackBar    db 'msctls_trackbar32',0
  Spin        db 'msctls_updown32',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 248

  NewPos      dd ?

  hWnd        dd ?
  hFontMSSS   dd ?

  wc          WNDCLASS   0, WindowProc, 0, 0, 0, 0, 0, COLOR_BTNFACE+1, 0, ClassName
  msg         MSG
  icex        INITCOMMONCONTROLSEX

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL',\
          comctl32,'COMCTL32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'
  include 'apia\comctl32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================
